@@ -50,6 +50,7 @@ func init() {
 	downloadAppledbCmd.Flags().StringArray("os", []string{}, fmt.Sprintf("Operating system to download (%s)", strings.Join(supportedOSes, ", ")))
 	downloadAppledbCmd.Flags().String("type", "ipsw", fmt.Sprintf("FW type to download (%s)", strings.Join(supportedFWs, ", ")))
 	downloadAppledbCmd.Flags().Bool("kernel", false, "Extract kernelcache from remote IPSW")
+	downloadAppledbCmd.Flags().Bool("dyld", false, "Extract dyld_shared_cache(s) from remote OTA")
 	downloadAppledbCmd.Flags().String("pattern", "", "Download remote files that match regex")
 	downloadAppledbCmd.Flags().Bool("beta", false, "Download beta IPSWs")
 	downloadAppledbCmd.Flags().Bool("latest", false, "Download latest IPSWs")
@@ -65,6 +66,7 @@ func init() {
 	viper.BindPFlag("download.appledb.os", downloadAppledbCmd.Flags().Lookup("os"))
 	viper.BindPFlag("download.appledb.type", downloadAppledbCmd.Flags().Lookup("type"))
 	viper.BindPFlag("download.appledb.kernel", downloadAppledbCmd.Flags().Lookup("kernel"))
+	viper.BindPFlag("download.appledb.dyld", downloadAppledbCmd.Flags().Lookup("dyld"))
 	viper.BindPFlag("download.appledb.pattern", downloadAppledbCmd.Flags().Lookup("pattern"))
 	viper.BindPFlag("download.appledb.beta", downloadAppledbCmd.Flags().Lookup("beta"))
 	viper.BindPFlag("download.appledb.latest", downloadAppledbCmd.Flags().Lookup("latest"))
@@ -143,6 +145,7 @@ var downloadAppledbCmd = &cobra.Command{
 		osTypes := viper.GetStringSlice("download.appledb.os")
 		fwType := viper.GetString("download.appledb.type")
 		kernel := viper.GetBool("download.appledb.kernel")
+		dyld := viper.GetBool("download.appledb.dyld")
 		pattern := viper.GetString("download.appledb.pattern")
 		isBeta := viper.GetBool("download.appledb.beta")
 		latest := viper.GetBool("download.appledb.latest")
@@ -300,7 +303,7 @@ var downloadAppledbCmd = &cobra.Command{
 		}
 
 		if cont {
-			if kernel || len(pattern) > 0 {
+			if kernel || dyld || len(pattern) > 0 {
 				for _, result := range results {
 					var url string
 					for _, link := range result.Links {
@@ -308,8 +311,13 @@ var downloadAppledbCmd = &cobra.Command{
 							url = link.URL
 						}
 					}
-					d, v, b := download.ParseIpswURLString(url)
-					log.WithFields(log.Fields{"devices": d, "build": b, "version": v}).Info("Parsing remote IPSW")
+					switch fwType {
+					case "ipsw":
+						d, v, b := download.ParseIpswURLString(url)
+						log.WithFields(log.Fields{"devices": d, "build": b, "version": v}).Info("Parsing remote IPSW")
+					case "ota", "rsr":
+						log.WithFields(log.Fields{"devices": device, "build": build, "version": version}).Info("Parsing remote OTA")
+					}
 
 					config := &extract.Config{
 						URL:          url,
@@ -344,6 +352,20 @@ var downloadAppledbCmd = &cobra.Command{
 							}
 						}
 					}
+					// REMOTE DSC MODE
+					if dyld {
+						if fwType != "ota" {
+							return fmt.Errorf("dyld_shared_cache(s) can only be extracted from OTA files (for now)")
+						}
+						log.Info("Extracting remote dyld_shared_cache(s)")
+						if out, err := extract.DSC(config); err != nil {
+							return err
+						} else {
+							for _, f := range out {
+								utils.Indent(log.Info, 2)("Created " + f)
+							}
+						}
+					}
 				}
 			} else { // NORMAL MODE
 				downloader := download.NewDownload(proxy, insecure, skipAll, resumeAll, restartAll, false, viper.GetBool("verbose"))
