@@ -4,22 +4,28 @@ import (
 	"bytes"
 	"compress/gzip"
 	_ "embed"
+	"encoding/base64"
 	"encoding/json"
 	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
 	"regexp"
+	"slices"
 	"sort"
 	"strings"
 	"text/tabwriter"
 	"time"
 
 	"github.com/apex/log"
+	"github.com/blacktop/arm64-cgo/disassemble"
 	"github.com/blacktop/go-macho"
 	"github.com/blacktop/go-macho/types"
 	"github.com/blacktop/ipsw/internal/commands/dsc"
+	"github.com/blacktop/ipsw/internal/demangle"
 	"github.com/blacktop/ipsw/internal/search"
+	"github.com/blacktop/ipsw/internal/swift"
+	"github.com/blacktop/ipsw/pkg/disass"
 	"github.com/fatih/color"
 )
 
@@ -34,7 +40,7 @@ var colorTime = color.New(color.Bold, color.FgHiGreen).SprintFunc()
 var colorError = color.New(color.Bold, color.FgHiRed).SprintFunc()
 var colorAddr = color.New(color.Faint).SprintfFunc()
 var colorBold = color.New(color.Bold).SprintfFunc()
-var colorImage = color.New(color.Bold, color.FgHiMagenta).SprintFunc()
+var colorImage = color.New(color.Bold, color.FgHiMagenta).SprintfFunc()
 var colorField = color.New(color.Bold, color.FgHiBlue).SprintFunc()
 
 var ErrDone = errors.New("done")
@@ -81,9 +87,17 @@ func GetLogTypes() (*LogTypes, error) {
 	return &db.LogTypes, nil
 }
 
+type Config struct {
+	All      bool
+	Running  bool
+	Unslid   bool
+	Demangle bool
+}
+
 type Ips struct {
 	Header  IpsMetadata
 	Payload IPSPayload
+	Config  *Config
 }
 
 type Platform int
@@ -186,6 +200,49 @@ type MemoryStatus struct {
 	PageSize int64 `json:"pageSize,omitempty"`
 }
 
+func (ms MemoryStatus) String() string {
+	return fmt.Sprintf(
+		"%s:\n"+
+			"  %s: %d\n"+
+			"  %s: %d\n"+
+			"  %s: %d\n"+
+			"  %s: %d\n"+
+			"  %s:\n"+
+			"    %s: %d\n"+
+			"    %s: %d\n"+
+			"    %s: %d\n"+
+			"    %s: %d\n"+
+			"    %s: %d\n"+
+			"    %s: %d\n"+
+			"    %s: %d\n"+
+			"    %s: %d\n"+
+			"  %s: %t\n"+
+			"  %s:\n"+
+			"    %s: %d\n"+
+			"    %s: %d\n"+
+			"  %s: %d\n",
+		colorField("Memory Status"),
+		colorField("Busy Buffer Count"), ms.BusyBufferCount,
+		colorField("Compressions"), ms.Compressions,
+		colorField("Compressor Size"), ms.CompressorSize,
+		colorField("Decompressions"), ms.Decompressions,
+		colorField("Memory Pages"),
+		colorField("Active"), ms.MemoryPages.Active,
+		colorField("File Backed"), ms.MemoryPages.FileBacked,
+		colorField("Free"), ms.MemoryPages.Free,
+		colorField("Inactive"), ms.MemoryPages.Inactive,
+		colorField("Purgeable"), ms.MemoryPages.Purgeable,
+		colorField("Speculative"), ms.MemoryPages.Speculative,
+		colorField("Throttled"), ms.MemoryPages.Throttled,
+		colorField("Wired"), ms.MemoryPages.Wired,
+		colorField("Memory Pressure"), ms.MemoryPressure,
+		colorField("Memory Pressure Details"),
+		colorField("Pages Reclaimed"), ms.MemoryPressureDetails.PagesReclaimed,
+		colorField("Pages Wanted"), ms.MemoryPressureDetails.PagesWanted,
+		colorField("Page Size"), ms.PageSize,
+	)
+}
+
 type Process struct {
 	ID                  int            `json:"pid"`
 	Name                string         `json:"procname"`
@@ -232,6 +289,7 @@ type BinaryImage struct {
 	Size   uint64 `json:"size,omitempty"`
 	Source string `json:"source,omitempty"`
 	UUID   string `json:"uuid,omitempty"`
+	Slide  uint64 `json:"slide,omitempty"`
 }
 
 func (bi *BinaryImage) UnmarshalJSON(b []byte) error {
@@ -280,12 +338,17 @@ type BundleInfo struct {
 	CFBundleIdentifier         string `json:"CFBundleIdentifier,omitempty"`
 	CFBundleShortVersionString string `json:"CFBundleShortVersionString,omitempty"`
 	CFBundleVersion            string `json:"CFBundleVersion,omitempty"`
+	DTAppStoreToolsBuild       string `json:"DTAppStoreToolsBuild,omitempty"`
 }
 
 type StoreInfo struct {
-	ApplicationVariant        string `json:"applicationVariant,omitempty"`
-	DeviceIdentifierForVendor string `json:"deviceIdentifierForVendor,omitempty"`
-	ItemID                    string `json:"itemID,omitempty"`
+	ItemID                            string `json:"itemID,omitempty"`
+	StoreCohortMetadata               string `json:"storeCohortMetadata,omitempty"`
+	DistributorID                     string `json:"distributorID,omitempty"`
+	ApplicationVariant                string `json:"applicationVariant,omitempty"`
+	DeviceIdentifierForVendor         string `json:"deviceIdentifierForVendor,omitempty"`
+	SoftwareVersionExternalIdentifier string `json:"softwareVersionExternalIdentifier,omitempty"`
+	ThirdParty                        bool   `json:"thirdParty,omitempty"`
 }
 
 type Exception struct {
@@ -367,7 +430,8 @@ type Frame struct {
 	ImageIndex     uint64 `json:"imageIndex,omitempty"`
 	ImageOffset    uint64 `json:"imageOffset,omitempty"`
 	Symbol         string `json:"symbol,omitempty"`
-	SymbolLocation int    `json:"symbolLocation,omitempty"`
+	SymbolLocation uint64 `json:"symbolLocation,omitempty"`
+	Slide          uint64 `json:"slide,omitempty"`
 }
 
 type PanicFrame struct {
@@ -375,7 +439,8 @@ type PanicFrame struct {
 	ImageName      string `json:"imageName,omitempty"`
 	ImageOffset    uint64 `json:"imageOffset,omitempty"`
 	Symbol         string `json:"symbol,omitempty"`
-	SymbolLocation int    `json:"symbolLocation,omitempty"`
+	SymbolLocation uint64 `json:"symbolLocation,omitempty"`
+	Slide          uint64 `json:"slide,omitempty"`
 }
 
 func (pf *PanicFrame) UnmarshalJSON(b []byte) error {
@@ -421,42 +486,52 @@ type IPSPayload struct {
 	Asi map[string][]string `json:"asi,omitempty"` // Additional application-specific logging. The properties of this object include an array of log strings.
 	// For more information, see Diagnostic messages. This appears in a translated report under Application Specific Information.
 	// https://developer.apple.com/documentation/xcode/examining-the-fields-in-a-crash-report#Diagnostic-messages
-	IsCorpse               bool         `json:"isCorpse,omitempty"`
-	IsNonFatal             string       `json:"isNonFatal,omitempty"`
-	IsSimulated            string       `json:"isSimulated,omitempty"`
-	Uptime                 int          `json:"uptime,omitempty"`
-	Translated             bool         `json:"translated,omitempty"`
-	ProcName               string       `json:"procName,omitempty"`
-	ProcPath               string       `json:"procPath,omitempty"`
-	ProcRole               string       `json:"procRole,omitempty"`
-	UserID                 int          `json:"userID,omitempty"`
-	DeployVersion          int          `json:"deployVersion,omitempty"`
-	ModelCode              string       `json:"modelCode,omitempty"`
-	CoalitionID            int          `json:"coalitionID,omitempty"`
-	OsVersion              OsVersion    `json:"osVersion,omitempty"`
-	CaptureTime            string       `json:"captureTime,omitempty"`
-	PID                    int          `json:"pid,omitempty"`
-	CPUType                string       `json:"cpuType,omitempty"`
-	RootsInstalled         int          `json:"roots_installed,omitempty"`
-	BugType                string       `json:"bug_type,omitempty"`
-	ProcLaunch             string       `json:"procLaunch,omitempty"`
-	ProcStartAbsTime       int64        `json:"procStartAbsTime,omitempty"`
-	ProcExitAbsTime        int64        `json:"procExitAbsTime,omitempty"`
-	BundleInfo             BundleInfo   `json:"bundleInfo,omitempty"`
-	StoreInfo              StoreInfo    `json:"storeInfo,omitempty"`
-	ParentProc             string       `json:"parentProc,omitempty"`
-	ParentPid              int          `json:"parentPid,omitempty"`
-	CoalitionName          string       `json:"coalitionName,omitempty"`
-	LockdownMode           int          `json:"ldm,omitempty"`
-	WasUnlockedSinceBoot   int          `json:"wasUnlockedSinceBoot,omitempty"`
-	IsLocked               int          `json:"isLocked,omitempty"`
-	ThrottleTimeout        int64        `json:"throttleTimeout,omitempty"`
-	BasebandVersion        string       `json:"basebandVersion,omitempty"`
-	Exception              Exception    `json:"exception,omitempty"`
-	LastExceptionBacktrace []Frame      `json:"lastExceptionBacktrace,omitempty"`
-	FaultingThread         int          `json:"faultingThread,omitempty"`
-	Threads                []UserThread `json:"threads,omitempty"`
-	UsedImages             []struct {
+	IsCorpse              bool       `json:"isCorpse,omitempty"`
+	IsNonFatal            string     `json:"isNonFatal,omitempty"`
+	IsSimulated           string     `json:"isSimulated,omitempty"`
+	Uptime                int        `json:"uptime,omitempty"`
+	Translated            bool       `json:"translated,omitempty"`
+	ProcName              string     `json:"procName,omitempty"`
+	ProcPath              string     `json:"procPath,omitempty"`
+	ProcRole              string     `json:"procRole,omitempty"`
+	ProcLaunch            string     `json:"procLaunch,omitempty"`
+	ProcStartAbsTime      int64      `json:"procStartAbsTime,omitempty"`
+	ProcExitAbsTime       int64      `json:"procExitAbsTime,omitempty"`
+	UserID                int        `json:"userID,omitempty"`
+	DeployVersion         int        `json:"deployVersion,omitempty"`
+	ModelCode             string     `json:"modelCode,omitempty"`
+	CoalitionID           int        `json:"coalitionID,omitempty"`
+	OsVersion             OsVersion  `json:"osVersion,omitempty"`
+	CaptureTime           string     `json:"captureTime,omitempty"`
+	PID                   int        `json:"pid,omitempty"`
+	CPUType               string     `json:"cpuType,omitempty"`
+	RootsInstalled        int        `json:"roots_installed,omitempty"`
+	BugType               string     `json:"bug_type,omitempty"`
+	BundleInfo            BundleInfo `json:"bundleInfo,omitempty"`
+	StoreInfo             StoreInfo  `json:"storeInfo,omitempty"`
+	ParentProc            string     `json:"parentProc,omitempty"`
+	ParentPid             int        `json:"parentPid,omitempty"`
+	CoalitionName         string     `json:"coalitionName,omitempty"`
+	LockdownMode          int        `json:"ldm,omitempty"`
+	WasUnlockedSinceBoot  int        `json:"wasUnlockedSinceBoot,omitempty"`
+	IsLocked              int        `json:"isLocked,omitempty"`
+	InstructionByteStream struct {
+		BeforePC string `json:"beforePC,omitempty"`
+		AtPC     string `json:"atPC,omitempty"`
+	} `json:"instructionByteStream,omitempty"`
+	CodeSigningID                 string       `json:"codeSigningID,omitempty"`
+	CodeSigningTeamID             string       `json:"codeSigningTeamID,omitempty"`
+	CodeSigningFlags              int          `json:"codeSigningFlags,omitempty"`
+	CodeSigningValidationCategory int          `json:"codeSigningValidationCategory,omitempty"`
+	CodeSigningTrustLevel         int          `json:"codeSigningTrustLevel,omitempty"`
+	CodeSigningMonitor            int          `json:"codeSigningMonitor,omitempty"`
+	ThrottleTimeout               int64        `json:"throttleTimeout,omitempty"`
+	BasebandVersion               string       `json:"basebandVersion,omitempty"`
+	Exception                     Exception    `json:"exception,omitempty"`
+	LastExceptionBacktrace        []Frame      `json:"lastExceptionBacktrace,omitempty"`
+	FaultingThread                int          `json:"faultingThread,omitempty"`
+	Threads                       []UserThread `json:"threads,omitempty"`
+	UsedImages                    []struct {
 		Arch   string `json:"arch,omitempty"`
 		Base   uint64 `json:"base,omitempty"`
 		Name   string `json:"name,omitempty"`
@@ -521,8 +596,8 @@ func ParseHeader(in string) (hdr *IpsMetadata, err error) {
 	return hdr, nil
 }
 
-func OpenIPS(in string) (*Ips, error) {
-	var ips Ips
+func OpenIPS(in string, conf *Config) (*Ips, error) {
+	ips := Ips{Config: conf}
 
 	f, err := os.Open(in)
 	if err != nil {
@@ -567,6 +642,9 @@ func (i *Ips) Symbolicate210(ipswPath string) error {
 		case "Absolute":
 			i.Payload.BinaryImages[idx].Name = "absolute"
 			total--
+		case "Kernel":
+			i.Payload.BinaryImages[idx].Name = "kernel"
+			total--
 		case "KernelCache":
 			i.Payload.BinaryImages[idx].Name = "kernelcache"
 			total--
@@ -588,23 +666,22 @@ func (i *Ips) Symbolicate210(ipswPath string) error {
 			return ErrDone // break
 		}
 		for idx, img := range i.Payload.BinaryImages {
-			var slide uint64
 			if m.UUID() != nil && strings.EqualFold(img.UUID, m.UUID().UUID.String()) {
 				i.Payload.BinaryImages[idx].Path = path
 				i.Payload.BinaryImages[idx].Name = path
-				slide = i.Payload.BinaryImages[idx].Base - m.GetBaseAddress()
 				// i.Payload.BinaryImages[idx].Name = filepath.Base(path)
+				i.Payload.BinaryImages[idx].Slide = i.Payload.BinaryImages[idx].Base - m.GetBaseAddress()
 				for _, fn := range m.GetFunctions() {
 					if syms, err := m.FindAddressSymbols(fn.StartAddr); err == nil {
 						for _, sym := range syms {
 							fn.Name = sym.Name
 						}
-						fn.StartAddr += slide
-						fn.EndAddr += slide
+						fn.StartAddr += i.Payload.BinaryImages[idx].Slide
+						fn.EndAddr += i.Payload.BinaryImages[idx].Slide
 						machoFuncMap[i.Payload.BinaryImages[idx].Name] = append(machoFuncMap[i.Payload.BinaryImages[idx].Name], fn)
 					} else {
-						fn.StartAddr += slide
-						fn.EndAddr += slide
+						fn.StartAddr += i.Payload.BinaryImages[idx].Slide
+						fn.EndAddr += i.Payload.BinaryImages[idx].Slide
 						fn.Name = fmt.Sprintf("func_%x", fn.StartAddr)
 						machoFuncMap[i.Payload.BinaryImages[idx].Name] = append(machoFuncMap[i.Payload.BinaryImages[idx].Name], fn)
 					}
@@ -635,14 +712,23 @@ func (i *Ips) Symbolicate210(ipswPath string) error {
 	for pid, proc := range i.Payload.ProcessByPid {
 		for tid, thread := range proc.ThreadByID {
 			for idx, frame := range thread.UserFrames {
+				i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset += i.Payload.BinaryImages[frame.ImageIndex].Base
+				if i.Payload.BinaryImages[frame.ImageIndex].Slide != 0 {
+					i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Slide = i.Payload.BinaryImages[frame.ImageIndex].Slide
+				}
 				if len(i.Payload.BinaryImages[frame.ImageIndex].Name) > 0 {
 					i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageName = i.Payload.BinaryImages[frame.ImageIndex].Name
+				} else {
+					if strings.HasPrefix(i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageName, "image_") {
+						i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageName += fmt.Sprintf(" (probably %s)", proc.Name)
+					}
 				}
 				if i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageName == "absolute" {
-					continue
-				}
-				i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset += i.Payload.BinaryImages[frame.ImageIndex].Base
-				if i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageName == "dyld_shared_cache" {
+					continue // skip absolute
+				} else if i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageName == "dyld_shared_cache" {
+					if f.Headers[f.UUID].SharedRegionStart != i.Payload.BinaryImages[frame.ImageIndex].Base {
+						i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Slide = i.Payload.BinaryImages[frame.ImageIndex].Base - f.Headers[f.UUID].SharedRegionStart
+					}
 					// lookup symbol in DSC dylib
 					if img, err := f.GetImageContainingVMAddr(i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset); err == nil {
 						i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageName = filepath.Base(img.Name)
@@ -654,11 +740,20 @@ func (i *Ips) Symbolicate210(ipswPath string) error {
 						}
 						if fn, err := m.GetFunctionForVMAddr(i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset); err == nil {
 							if sym, ok := f.AddressToSymbol[fn.StartAddr]; ok {
-								delta := ""
+								if i.Config.Demangle {
+									if strings.HasPrefix(sym, "_$s") || strings.HasPrefix(sym, "$s") { // TODO: better detect swift symbols
+										i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Symbol, _ = swift.Demangle(sym)
+									} else if strings.HasPrefix(sym, "__Z") || strings.HasPrefix(sym, "_Z") {
+										i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Symbol = demangle.Do(sym, false, false)
+									} else {
+										i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Symbol = sym
+									}
+								} else {
+									i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Symbol = sym
+								}
 								if i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset-fn.StartAddr != 0 {
-									delta = fmt.Sprintf(" + %d", i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset-fn.StartAddr)
+									i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].SymbolLocation = i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset - fn.StartAddr
 								}
-								i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Symbol = sym + delta
 							} else {
 								i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Symbol = fmt.Sprintf("func_%x", fn.StartAddr)
 							}
@@ -669,11 +764,21 @@ func (i *Ips) Symbolicate210(ipswPath string) error {
 					if funcs, ok := machoFuncMap[i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageName]; ok {
 						for _, fn := range funcs {
 							if fn.StartAddr <= i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset && i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset < fn.EndAddr {
-								delta := ""
-								if i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset-fn.StartAddr != 0 {
-									delta = fmt.Sprintf(" + %d", i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset-fn.StartAddr)
+								if i.Config.Demangle {
+									if strings.HasPrefix(fn.Name, "_$s") || strings.HasPrefix(fn.Name, "$s") { // TODO: better detect swift symbols
+										i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Symbol, _ = swift.Demangle(fn.Name)
+									} else if strings.HasPrefix(fn.Name, "__Z") || strings.HasPrefix(fn.Name, "_Z") {
+										i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Symbol = demangle.Do(fn.Name, false, false)
+									} else {
+										i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Symbol = fn.Name
+									}
+								} else {
+									i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Symbol = fn.Name
 								}
-								i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].Symbol = fn.Name + delta
+								if i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset-fn.StartAddr != 0 {
+									i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].SymbolLocation = i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset - fn.StartAddr
+								}
+								break
 							}
 						}
 					} else {
@@ -682,15 +787,23 @@ func (i *Ips) Symbolicate210(ipswPath string) error {
 							"thread": tid,
 							"frame":  idx,
 							"img":    i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageName,
-						}).Debugf("failed to find function for offset %#x", i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset)
+						}).Debugf("failed to find function for process offset %#x", i.Payload.ProcessByPid[pid].ThreadByID[tid].UserFrames[idx].ImageOffset)
 					}
 				}
 			}
 			for idx, frame := range thread.KernelFrames {
+				i.Payload.ProcessByPid[pid].ThreadByID[tid].KernelFrames[idx].ImageOffset += i.Payload.BinaryImages[frame.ImageIndex].Base
 				if len(i.Payload.BinaryImages[frame.ImageIndex].Name) > 0 {
 					i.Payload.ProcessByPid[pid].ThreadByID[tid].KernelFrames[idx].ImageName = i.Payload.BinaryImages[frame.ImageIndex].Name
+				} else {
+					if strings.HasPrefix(i.Payload.ProcessByPid[pid].ThreadByID[tid].KernelFrames[idx].ImageName, "image_") {
+						i.Payload.ProcessByPid[pid].ThreadByID[tid].KernelFrames[idx].ImageName += " (maybe a kext)"
+					}
+				}
+				if i.Payload.ProcessByPid[pid].ThreadByID[tid].KernelFrames[idx].ImageName == "kernelcache" {
+					// TODO: symbolicate kernelcache
+					// i.Payload.ProcessByPid[pid].ThreadByID[tid].KernelFrames[idx].Slide =
 				}
-				i.Payload.ProcessByPid[pid].ThreadByID[tid].KernelFrames[idx].ImageOffset += i.Payload.BinaryImages[frame.ImageIndex].Base
 			}
 		}
 	}
@@ -723,7 +836,7 @@ func colorVMSummary(in string) string {
 	return in
 }
 
-func (i *Ips) String(verbose bool) string {
+func (i *Ips) String() string {
 	var out string
 
 	switch i.Header.BugType {
@@ -735,6 +848,8 @@ func (i *Ips) String(verbose bool) string {
 		} else {
 			out += p210.String()
 		}
+		out += fmt.Sprintf("%s: %s\n", colorField("Panic Flags"), i.Payload.PanicFlags)
+		out += "\n" + i.Payload.MemoryStatus.String()
 		out += i.Payload.OtherString + "\n"
 		var pids []int
 		for pid := range i.Payload.ProcessByPid {
@@ -747,8 +862,23 @@ func (i *Ips) String(verbose bool) string {
 			if p.ID == p210.PanickedTask.PID {
 				paniced = colorError(" (Panicked)")
 			} else {
-				if !verbose {
-					continue
+				/* filter procs */
+				if !i.Config.All {
+					if i.Config.Running {
+						notRunning := true
+						// check if any thread is running
+						for _, t := range p.ThreadByID {
+							if slices.Contains(t.State, "TH_RUN") {
+								notRunning = false
+								break
+							}
+						}
+						if notRunning {
+							continue
+						}
+					} else {
+						continue
+					}
 				}
 			}
 			out += fmt.Sprintf(colorField("Process")+": %s [%s]%s\n", colorImage(p.Name), colorBold("%d", p.ID), paniced)
@@ -757,8 +887,15 @@ func (i *Ips) String(verbose bool) string {
 				if t.ID == p210.PanickedThread.TID {
 					paniced = colorError("       (Panicked)")
 				} else {
-					if !verbose {
-						continue
+					/* filter threads */
+					if !i.Config.All {
+						if i.Config.Running {
+							if !slices.Contains(t.State, "TH_RUN") {
+								continue
+							}
+						} else {
+							continue
+						}
 					}
 				}
 				out += fmt.Sprintf(colorField("  Thread")+": %s%s\n", colorBold("%d", t.ID), paniced)
@@ -778,11 +915,15 @@ func (i *Ips) String(verbose bool) string {
 					buf := bytes.NewBufferString("")
 					w := tabwriter.NewWriter(buf, 0, 0, 1, ' ', 0)
 					for idx, f := range t.UserFrames {
+						slide := ""
+						if f.Slide > 0 {
+							slide = fmt.Sprintf(" (slide %#x)", f.Slide)
+						}
 						symloc := ""
 						if f.SymbolLocation > 0 {
 							symloc = fmt.Sprintf(" + %d", f.SymbolLocation)
 						}
-						fmt.Fprintf(w, "      %02d: %s\t%s %s%s\n", idx, colorImage(f.ImageName), colorAddr("%#x", f.ImageOffset), colorField(f.Symbol), colorBold(symloc))
+						fmt.Fprintf(w, "      %02d: %s\t%s%s %s%s\n", idx, colorImage(f.ImageName), colorAddr("%#x", f.ImageOffset), slide, colorField(f.Symbol), symloc)
 					}
 					w.Flush()
 					out += buf.String()
@@ -792,11 +933,18 @@ func (i *Ips) String(verbose bool) string {
 					buf := bytes.NewBufferString("")
 					w := tabwriter.NewWriter(buf, 0, 0, 1, ' ', 0)
 					for idx, f := range t.KernelFrames {
+						slide := ""
+						if p210.KernelCacheSlide != nil && p210.KernelCacheSlide.Value.(uint64) > 0 && f.ImageName == "kernelcache" {
+							slide = fmt.Sprintf(" (slide %#x)", p210.KernelCacheSlide.Value.(uint64))
+						}
+						if p210.KernelSlide != nil && p210.KernelSlide.Value.(uint64) > 0 && (f.ImageName == "kernelcache" || f.ImageName == "kernel") {
+							slide = fmt.Sprintf(" (slide %#x)", p210.KernelSlide.Value.(uint64))
+						}
 						symloc := ""
 						if f.SymbolLocation > 0 {
 							symloc = fmt.Sprintf(" + %d", f.SymbolLocation)
 						}
-						fmt.Fprintf(w, "      %02d: %s\t%s %s%s\n", idx, colorImage(f.ImageName), colorAddr("%#x", f.ImageOffset), colorField(f.Symbol), colorBold(symloc))
+						fmt.Fprintf(w, "      %02d: %s\t%s%s %s%s\n", idx, colorImage(f.ImageName), colorAddr("%#x", f.ImageOffset), slide, colorField(f.Symbol), symloc)
 					}
 					w.Flush()
 					out += buf.String()
@@ -813,16 +961,25 @@ func (i *Ips) String(verbose bool) string {
 	case "Crash", "309":
 		out = fmt.Sprintf("[%s] - %s\n\n", colorTime(i.Header.Timestamp.Format("02Jan2006 15:04:05")), colorError(i.Header.BugTypeDesc))
 		out += fmt.Sprintf(
-			colorField("Process")+":             %s [%d]\n"+
-				colorField("Hardware Model")+":      %s\n"+
-				colorField("OS Version")+":          %s\n"+
-				colorField("BuildID")+":             %s\n"+
-				colorField("LockdownMode")+":        %d\n",
+			colorField("Process")+":        %s [%d]\n"+
+				colorField("Path")+":           %s\n"+
+				colorField("Parent")+":         %s [%d]\n"+
+				colorField("Hardware Model")+": %s\n"+
+				colorField("OS Version")+":     %s\n"+
+				colorField("BuildID")+":        %s\n"+
+				colorField("LockdownMode")+":            %d\n"+
+				colorField("Was Unlocked Since Boot")+": %d\n"+
+				colorField("Is Locked")+":               %d\n",
 			i.Payload.ProcName, i.Payload.PID,
+			i.Payload.ProcPath,
+			i.Payload.ParentProc, i.Payload.ParentPid,
 			i.Payload.ModelCode,
 			i.Payload.OsVersion.Train,
 			i.Payload.OsVersion.Build,
-			i.Payload.LockdownMode)
+			i.Payload.LockdownMode,
+			i.Payload.WasUnlockedSinceBoot,
+			i.Payload.IsLocked,
+		)
 		if i.Payload.SharedCache.Size > 0 {
 			out += fmt.Sprintf(colorField("Shared Cache")+":        %s %s: %#x %s: %d\n", i.Payload.SharedCache.UUID, colorField("base"), i.Payload.SharedCache.Base, colorField("size"), i.Payload.SharedCache.Size)
 		}
@@ -867,13 +1024,65 @@ func (i *Ips) String(verbose bool) string {
 					if f.SymbolLocation > 0 {
 						symloc = fmt.Sprintf(" + %d", f.SymbolLocation)
 					}
-					fmt.Fprintf(w, "  %02d: %s\t%s %s%s\n", idx, colorImage(i.Payload.UsedImages[f.ImageIndex].Name), colorAddr("%#x", i.Payload.UsedImages[f.ImageIndex].Base+f.ImageOffset), colorField(f.Symbol), colorBold(symloc))
+					fmt.Fprintf(w, "  %02d: %s\t%s %s%s\n", idx, colorImage(i.Payload.UsedImages[f.ImageIndex].Name), colorAddr("%#x", i.Payload.UsedImages[f.ImageIndex].Base+f.ImageOffset), colorField(f.Symbol), symloc)
 				}
 				w.Flush()
 				out += buf.String()
 			}
 			if t.Triggered {
 				out += colorField("Thread") + colorBold(" %d ", t.ID) + colorField(t.ThreadState.Flavor) + "\n" + t.ThreadState.String()
+				if len(i.Payload.InstructionByteStream.BeforePC) > 0 {
+					out += "\n"
+					b64data, err := base64.StdEncoding.WithPadding(base64.StdPadding).DecodeString(i.Payload.InstructionByteStream.BeforePC)
+					if err != nil {
+						log.WithError(err).Errorf("failed to decode instruction byte stream")
+					}
+					instructions, err := disassemble.GetInstructions(t.ThreadState.PC.Value-(10*4), b64data)
+					if err != nil {
+						log.WithError(err).Errorf("failed to disassemble instructions")
+					}
+					pad := "    "
+					for _, block := range instructions.Blocks() {
+						for _, i := range block {
+							opStr := strings.TrimSpace(strings.TrimPrefix(i.String(), i.Operation.String()))
+							out += fmt.Sprintf("%s%s:  %s   %s %s\n",
+								pad,
+								colorAddr("%#08x", i.Address),
+								disassemble.GetOpCodeByteString(i.Raw),
+								colorImage("%-7s", i.Operation),
+								disass.ColorOperands(opStr),
+							)
+						}
+						out += "\n"
+					}
+				}
+				if len(i.Payload.InstructionByteStream.AtPC) > 0 {
+					b64data, err := base64.StdEncoding.WithPadding(base64.StdPadding).DecodeString(i.Payload.InstructionByteStream.AtPC)
+					if err != nil {
+						log.WithError(err).Errorf("failed to decode instruction byte stream")
+					}
+					instructions, err := disassemble.GetInstructions(t.ThreadState.PC.Value, b64data)
+					if err != nil {
+						log.WithError(err).Errorf("failed to disassemble instructions")
+					}
+					for idx, block := range instructions.Blocks() {
+						pad := "    "
+						if idx == 0 {
+							pad = colorError("PC=>")
+						}
+						for _, i := range block {
+							opStr := strings.TrimSpace(strings.TrimPrefix(i.String(), i.Operation.String()))
+							out += fmt.Sprintf("%s%s:  %s   %s %s\n",
+								pad,
+								colorAddr("%#08x", i.Address),
+								disassemble.GetOpCodeByteString(i.Raw),
+								colorImage("%-7s", i.Operation),
+								disass.ColorOperands(opStr),
+							)
+						}
+						out += "\n"
+					}
+				}
 			}
 			out += "\n"
 		}
@@ -882,7 +1091,11 @@ func (i *Ips) String(verbose bool) string {
 			buf := bytes.NewBufferString("")
 			w := tabwriter.NewWriter(buf, 0, 0, 1, ' ', 0)
 			for idx, f := range i.Payload.LastExceptionBacktrace {
-				fmt.Fprintf(w, "  %02d: %s\t%s %s + %d\n", idx, colorImage(i.Payload.UsedImages[f.ImageIndex].Name), colorAddr("%#x", i.Payload.UsedImages[f.ImageIndex].Base+f.ImageOffset), colorField(f.Symbol), f.SymbolLocation)
+				symloc := ""
+				if f.SymbolLocation > 0 {
+					symloc = fmt.Sprintf(" + %d", f.SymbolLocation)
+				}
+				fmt.Fprintf(w, "  %02d: %s\t%s %s%s\n", idx, colorImage(i.Payload.UsedImages[f.ImageIndex].Name), colorAddr("%#x", i.Payload.UsedImages[f.ImageIndex].Base+f.ImageOffset), colorField(f.Symbol), symloc)
 			}
 			w.Flush()
 			out += buf.String() + "\n"
